package cn.com.ctop.crawler.modules.core.util;

import cn.com.ctop.crawler.modules.appium.entity.AppiumTaskItem;
import com.google.gson.Gson;
import io.appium.java_client.TouchAction;
import io.appium.java_client.android.AndroidDriver;
import io.appium.java_client.touch.WaitOptions;
import io.appium.java_client.touch.offset.PointOption;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.Point;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.remote.DesiredCapabilities;

import java.net.URL;
import java.time.Duration;
import java.util.List;

@Slf4j
public class AppiumUtil {
    private Boolean isLoop = true;
    private Duration duration = Duration.ofSeconds(1);

    public static final String FIND_ELEMENT_TYPE_ID = "id";
    public static final String FIND_ELEMENT_TYPE_XPATH = "xpath";
    public static final String FIND_ELEMENT_TYPE_CLASS = "class";
    public static final String APP_SWIPE_TYPE_UP = "up";
    public static final String APP_SWIPE_TYPE_DOWN = "down";
    public static final String APP_SWIPE_TYPE_LEFT = "left";
    public static final String APP_SWIPE_TYPE_RIGHT = "right";

    public void swipeToUp(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action1 = new TouchAction(driver).press(PointOption.point(width / 2, height * 3 / 4)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 2, height / 4)).release();
        action1.perform();
    }

    public void swipeToDown(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action2 = new TouchAction(driver).press(PointOption.point(width / 2, height / 4)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 2, height * 3 / 4)).release();
        action2.perform();
    }

    public void swipeToLeft(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action3 = new TouchAction(driver).press(PointOption.point(width * 3 / 4, height / 2)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width / 4, height / 2)).release();
        action3.perform();
    }

    public void tapPoint(AndroidDriver driver, Integer x, Integer y) {
        TouchAction action = new TouchAction(driver).tap(PointOption.point(x, y)).release();
        action.perform();
    }

    public void swipeToRight(AndroidDriver driver) {
        int width = driver.manage().window().getSize().width;
        int height = driver.manage().window().getSize().height;
        TouchAction action4 = new TouchAction(driver).press(PointOption.point(width / 4, height / 2)).waitAction(WaitOptions.waitOptions(duration))
                .moveTo(PointOption.point(width * 3 / 4, height / 2)).release();
        action4.perform();
    }

    public AndroidDriver initAndroidDriver(String serverUrl, String deviceName, String appPackage, String appActivity) throws Exception {
        return new AndroidDriver(new URL(serverUrl), initCapabilities(deviceName, appPackage, appActivity));
    }

    public DesiredCapabilities initCapabilities(String deviceName, String appPackage, String appActivity) {
        DesiredCapabilities desiredCapabilities = new DesiredCapabilities();
        desiredCapabilities.setCapability("deviceName", deviceName);
        desiredCapabilities.setCapability("platformName", "Android");
        desiredCapabilities.setCapability("appPackage", appPackage);
        desiredCapabilities.setCapability("noReset", true);
        desiredCapabilities.setCapability("unicodeKeyboard", true);
        desiredCapabilities.setCapability("resetKeyboard", true);
        desiredCapabilities.setCapability("appActivity", appActivity);
        return desiredCapabilities;
    }

    public static void main(String[] args) {
        AppiumUtil appiumUtil = new AppiumUtil();
        try {
            AndroidDriver androidDriver = appiumUtil.initAndroidDriver("http://39.106.184.70:4723/wd/hub", "183.131.157.66:10005", "com.smile.gifmaker", "com.yxcorp.gifshow.HomeActivity");
            appiumUtil.loginKuaishou(androidDriver);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void loginKuaishou(AndroidDriver androidDriver) {
        try {
            Thread.sleep(3000L);
            WebElement closeBtn1 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/close_iv");
            if (closeBtn1 != null) {
                closeBtn1.click();
            }
            for (int i = 0; i < 5; i++) {
                Thread.sleep(3000L);
                permissionAllow(androidDriver);
            }
            Thread.sleep(3000);
            WebElement continueBtn1 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/positive");
            if (continueBtn1 != null) {
                continueBtn1.click();
            }
            for (int i = 0; i < 3; i++) {
                Thread.sleep(3000L);
                closeBtnClick(androidDriver);
            }
            WebElement loginBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_text");
            System.out.println(loginBtn.getText());
            if (loginBtn != null && loginBtn.getText().equals("登录")) {
                loginBtn.click();
                Thread.sleep(3000L);

                WebElement mobileLoginBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/phone_login_view");
                if (mobileLoginBtn != null) {
                    mobileLoginBtn.click();
                    Thread.sleep(2000L);
                }
                WebElement mobileInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_name_et");
                WebElement loginTypeBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/phone_one_key_login_btn");
                WebElement mobileLogin = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/phone_et");
                if (mobileInput != null) {
                    String token = KuaimiUtil.kuaimiLogin();
                    String mobile = KuaimiUtil.getMobile(token, "2307");
                    mobileInput.sendKeys(mobile);
                    Thread.sleep(2000L);
                    WebElement nextBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/next_view");
                    nextBtn.click();
                    Thread.sleep(5000L);
                    WebElement pintu = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/title_tv");
                    if (pintu != null && pintu.getText().equals("拖动滑块")) {
                        WebElement leftBtn1 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_btn");
                        leftBtn1.click();
                        return;
                    }
                    WebElement loginTitle = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_title_tv");
                    WebElement forgetBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/forget_psd_btn");
                    WebElement captcha = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_et");
                    if (loginTitle != null && loginTitle.getText().equals("输入密码")) {
                        forgetBtn.click();
                        Thread.sleep(2000L);
                        WebElement sendMsgBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/verify_tv");
                        sendMsgBtn.click();
                        Thread.sleep(5000L);
                        int i = 0;
                        String msg = null;
                        while (i < 20) {
                            try {
                                msg = KuaimiUtil.getMessage(token, "2307", mobile);
                            } catch (Exception e) {
                                System.out.println("等待" + mobile + "短信验证码");
                            }
                            if (msg != null) {
                                WebElement captchrInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/verify_et");
                                captchrInput.sendKeys(msg);
                                Thread.sleep(2000);
                                WebElement confirmBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/confirm_btn");
                                confirmBtn.click();
                                Thread.sleep(2000);
                                WebElement resetSelect = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/avatar_layout_one");
                                if (resetSelect != null) {
                                    resetSelect.click();
                                    Thread.sleep(2000);
                                }
                                WebElement newPwdInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_psd_et");
                                newPwdInput.sendKeys("a123456");
                                Thread.sleep(2000);
                                WebElement confirmBtn2 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/confirm_btn");
                                confirmBtn2.click();
                                Thread.sleep(3000);
                                WebElement continueBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/positive");
                                if (continueBtn != null) {
                                    continueBtn.click();
                                }
                                for (int j = 0; j < 3; j++) {
                                    Thread.sleep(3000);
                                    WebElement leftBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_btn");
                                    if (leftBtn != null) {
                                        leftBtn.click();
                                    }
                                }

                                Thread.sleep(3000);
                                WebElement headImg = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/tab_avatar");
                                if (headImg != null) {
                                    headImg.click();
                                    Thread.sleep(3000);
                                    tapPoint(androidDriver, 650, 400);
                                    Thread.sleep(3000);
//                                    WebElement qrCode = getElementById(androidDriver,"com.smile.gifmaker:id/qr_code_image");
//                                    qrCode.click();
                                    WebElement kwid = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/kwai_id");
                                    System.out.println(kwid.getText());
                                }

                                break;
                            }
                            i++;
                            Thread.sleep(5000);
                        }
                        if (i == 20) {
                            KuaimiUtil.addBlackList(token, "2307", mobile);
                        }
                    } else if (captcha != null) {
                        int i = 0;
                        String msg = null;
                        while (i < 20) {

                            try {
                                msg = KuaimiUtil.getMessage(token, "2307", mobile);
                            } catch (Exception e) {
                                System.out.println("等待" + mobile + "短信验证码");
                            }
                            if (msg != null) {
                                Thread.sleep(2000);
                                WebElement captchaInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_et");
                                captchaInput.sendKeys(msg);
                                WebElement captchaConfirm = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_finish");
                                Thread.sleep(5000);
                                captchaConfirm.click();
                                System.out.println(msg);
                                Thread.sleep(5000);
                                WebElement icon = getElementByType(androidDriver, FIND_ELEMENT_TYPE_XPATH, "//android.widget.ImageView[@content-desc=\"头像\"]");
                                if (icon != null) {
                                    icon.click();
                                }
                                Thread.sleep(3000);
                                WebElement continueBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/positive");
                                if (continueBtn != null) {
                                    continueBtn.click();
                                }
                                for (int j = 0; j < 3; j++) {
                                    Thread.sleep(3000);
                                    WebElement leftBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_btn");
                                    if (leftBtn != null) {
                                        leftBtn.click();
                                    }
                                }
                                Thread.sleep(3000);
                                WebElement headImg = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/tab_avatar");
                                if (headImg != null) {
                                    headImg.click();
                                    Thread.sleep(3000);
                                    tapPoint(androidDriver, 650, 400);
                                    Thread.sleep(3000);
//                                    WebElement qrCode = getElementById(androidDriver,"com.smile.gifmaker:id/qr_code_image");
//                                    qrCode.click();
                                    WebElement kwid = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/kwai_id");
                                    System.out.println(kwid.getText());
                                }
                                //封禁
                                break;
                            }
                            i++;
                            Thread.sleep(5000);
                        }
                        if (i == 20) {
                            KuaimiUtil.addBlackList(token, "2307", mobile);
                        }
                    }
                }
                if (loginTypeBtn != null) {
                    loginTypeBtn.click();
                    Thread.sleep(5000);
                    mobileInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_name_et");
                    if (mobileInput != null) {
                        String token = KuaimiUtil.kuaimiLogin();
                        String mobile = KuaimiUtil.getMobile(token, "2307");
                        mobileInput.sendKeys(mobile);
                        Thread.sleep(2000L);
                        WebElement nextBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/next_view");
                        nextBtn.click();
                        Thread.sleep(5000L);
                        WebElement pintu = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/title_tv");
                        if (pintu != null && pintu.getText().equals("拖动滑块")) {
                            WebElement leftBtn1 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_btn");
                            leftBtn1.click();
                            return;
                        }
                        WebElement loginTitle = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_title_tv");
                        WebElement forgetBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/forget_psd_btn");
                        WebElement captcha = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_et");
                        if (loginTitle != null && loginTitle.getText().equals("输入密码")) {
                            forgetBtn.click();
                            Thread.sleep(2000L);
                            WebElement sendMsgBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/verify_tv");
                            sendMsgBtn.click();
                            Thread.sleep(5000L);
                            int i = 0;
                            String msg = null;
                            while (i < 20) {
                                try {
                                    msg = KuaimiUtil.getMessage(token, "2307", mobile);
                                } catch (Exception e) {
                                    System.out.println("等待" + mobile + "短信验证码");
                                }
                                if (msg != null) {
                                    WebElement captchrInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/verify_et");
                                    captchrInput.sendKeys(msg);
                                    Thread.sleep(5000);
                                    WebElement confirmBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/confirm_btn");
                                    confirmBtn.click();
                                    Thread.sleep(5000);
                                    WebElement newPwdInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/login_psd_et");
                                    newPwdInput.sendKeys("a123456");
                                    Thread.sleep(5000);
                                    WebElement confirmBtn2 = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/confirm_btn");
                                    confirmBtn2.click();
                                    Thread.sleep(3000);
                                    WebElement continueBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/positive");
                                    if (continueBtn != null) {
                                        continueBtn.click();
                                    }
                                    for (int j = 0; i < 3; j++) {
                                        Thread.sleep(3000);
                                        WebElement leftBtn = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/left_btn");
                                        leftBtn.click();
                                    }

                                    WebElement headImg = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/tab_avatar");
                                    if (headImg != null) {
                                        headImg.click();
                                        Thread.sleep(3000);
                                        WebElement qrCode = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/qr_code_image");
                                        qrCode.click();
                                        WebElement kwid = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/kwai_id");
                                        System.out.println(kwid.getText());
                                    }
                                    //封禁判断
                                    break;
                                }
                                i++;
                                Thread.sleep(5000);
                            }
                            if (i == 20) {
                                KuaimiUtil.addBlackList(token, "2307", mobile);
                            }
                        } else if (captcha != null) {
                            int i = 0;
                            String msg = null;
                            while (i < 20) {

                                try {
                                    msg = KuaimiUtil.getMessage(token, "2307", mobile);
                                } catch (Exception e) {
                                    System.out.println("等待" + mobile + "短信验证码");
                                }
                                if (msg != null) {
                                    WebElement captchaInput = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_et");
                                    captchaInput.sendKeys(msg);
                                    WebElement captchaConfirm = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/captcha_finish");
                                    Thread.sleep(2000);
                                    captchaConfirm.click();
                                    System.out.println(msg);
                                    Thread.sleep(5000);
                                    WebElement icon = getElementByType(androidDriver, FIND_ELEMENT_TYPE_XPATH, "//android.widget.ImageView[@content-desc=\"头像\"]");
                                    if (icon != null) {
                                        icon.click();
                                    }
                                }
                                i++;
                                Thread.sleep(5000);
                            }
                        }
                    }
                }
                if (mobileLogin != null) {
                    String token = KuaimiUtil.kuaimiLogin();
                    String mobile = KuaimiUtil.getMobile(token, "2307");
                    mobileLogin.sendKeys(mobile);
                    Thread.sleep(5000L);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            androidDriver.quit();
        }

    }

    public void permissionAllow(AndroidDriver androidDriver) {
        WebElement element = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.android.packageinstaller:id/permission_allow_button");
        if (element != null) {
            element.click();
        }
    }

    public void closeBtnClick(AndroidDriver androidDriver) {
        WebElement element = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/close_btn");
        if (element != null) {
            element.click();
        }
    }

    public void backToMain(AndroidDriver androidDriver) {
        WebElement element = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/back_btn");
        if (null != element) {
            element.click();
        }
    }

    public Boolean checkAdvertiser(AndroidDriver androidDriver, String advertiserName) {
        WebElement label = getElementByType(androidDriver, FIND_ELEMENT_TYPE_ID, "com.smile.gifmaker:id/label");
        String text = label.getText();
        if (null != label && null != text && text.startsWith(advertiserName)) {
            //表示广告筛选成功
            return true;
        } else {
            backToMain(androidDriver);
            return false;
        }
    }

    public WebElement getElementByType(AndroidDriver androidDriver, String type, String typeInfo) {
        WebElement element = null;
        try {
            if (type.equals(FIND_ELEMENT_TYPE_ID)) {
                element = androidDriver.findElementById(typeInfo);
            }
            if (type.equals(FIND_ELEMENT_TYPE_CLASS)) {
                element = androidDriver.findElementByClassName(typeInfo);
            }
            if (type.equals(FIND_ELEMENT_TYPE_XPATH)) {
                element = androidDriver.findElementByXPath(typeInfo);
            }
        } catch (Exception e) {
            log.error("元素不存在：findType：{}，keywords:{}", type, typeInfo);
        }
        return element;
    }

    public void runTask(AndroidDriver androidDriver, AppiumTaskItem appiumTaskItem, WebElement loopElement) throws Exception {
        if (appiumTaskItem != null) {
            System.out.println("开始任务" + appiumTaskItem.getId() + ":" + new Gson().toJson(appiumTaskItem));
            if (appiumTaskItem.getWaitTime() != null && appiumTaskItem.getWaitTime() > 0) {
                Thread.sleep(appiumTaskItem.getWaitTime());
            }
            if (appiumTaskItem.getLoopType() != null && appiumTaskItem.getLoopType().equals("noloop")) {
                //非循环类型任务
                if (appiumTaskItem.getFindType() != null) {
                    WebElement element;
                    //1：获取web元素
                    if (appiumTaskItem.getFindType().equals("loop")) {
                        element = loopElement;
                    } else {
                        element = getElementByType(androidDriver, appiumTaskItem.getFindType(), appiumTaskItem.getFindKey());
                    }
                    if (element != null) {
                        if (appiumTaskItem.getTextEqualType() != null) {
                            if (appiumTaskItem.getTextEqualType().equals("eq")) {
                                if (!element.getText().equals(appiumTaskItem.getTextEqualKey())) {
                                    return;
                                }
                            } else if (appiumTaskItem.getTextEqualType().equals("neq")) {
                                if (element.getText() == null || element.getText().equals(appiumTaskItem.getTextEqualKey())) {
                                    return;
                                }
                            } else if (appiumTaskItem.getTextEqualType().equals("start")) {
                                boolean check = checkAdvertiser(androidDriver, appiumTaskItem.getTextEqualKey());
                                if (!check) {
                                    return;
                                }
                            }
                        }
                        if (appiumTaskItem.getClickType() != null) {
                            if (appiumTaskItem.getClickType().equals("element")) {
                                element.click();
                            } else if (appiumTaskItem.getClickType().equals("point")) {
                                Point point = element.getLocation();
                                tapPoint(androidDriver, point.getX() + appiumTaskItem.getOffsiteX(), point.getY() + appiumTaskItem.getOffsiteY());
                            }
                        }
                        List<AppiumTaskItem> subTask = appiumTaskItem.getSubList();
                        if (subTask != null && subTask.size() > 0) {
                            for (AppiumTaskItem task : subTask) {
                                runTask(androidDriver, task, element);
                            }
                        }
                    } else {
                        //必须执行
                        if (appiumTaskItem.getIsMust() != null && appiumTaskItem.getIsMust() == 1) {
                            if (appiumTaskItem.getLoopRate() != null && appiumTaskItem.getLoopRate() > 0) {
                                Thread.sleep(appiumTaskItem.getLoopRate());
                            }
                            runTask(androidDriver, appiumTaskItem, element);
                            return;
                        }
                        //非必须执行情况下
                        if (appiumTaskItem.getClickType() != null && appiumTaskItem.getClickType().equals("point")) {
                            tapPoint(androidDriver, appiumTaskItem.getOffsiteX(), appiumTaskItem.getOffsiteY());
                        }
                    }
                }
                //屏幕滑动
                if (appiumTaskItem.getSwapType() != null) {
                    swipeByType(androidDriver, appiumTaskItem.getSwapType());
                }
            } else if (appiumTaskItem.getLoopType() != null && appiumTaskItem.getLoopType().equals("loop")) {
                //循环任务
                while (isLoop) {
                    System.out.println("loop:" + isLoop);
                    if (appiumTaskItem.getFindType() != null) {
                        List<WebElement> list = getElementsByType(androidDriver, appiumTaskItem.getFindType(), appiumTaskItem.getFindKey());
                        if (list != null && list.size() > 0) {
                            for (WebElement element : list) {
                                if (!isLoop) {
                                    break;
                                }
                                if (appiumTaskItem.getLoopRate() != null && appiumTaskItem.getLoopRate() > 0) {
                                    Thread.sleep(appiumTaskItem.getLoopRate());
                                }
                                List<AppiumTaskItem> subTask = appiumTaskItem.getSubList();
                                if (subTask != null && subTask.size() > 0) {
                                    for (AppiumTaskItem task : subTask) {
                                        runTask(androidDriver, task, element);
                                    }
                                }
                            }
                        }
                    }
                    if (appiumTaskItem.getSwapType() != null) {
                        swipeByType(androidDriver, appiumTaskItem.getSwapType());
                    }
                }

            }
            if (appiumTaskItem.getIsClose() != null && appiumTaskItem.getIsClose() == 1) {
                this.isLoop = false;
            }
            System.out.println("结束任务" + appiumTaskItem.getId() + ":" + new Gson().toJson(appiumTaskItem));
        }
    }

    private void swipeByType(AndroidDriver androidDriver, String type) {
        if (type.equals(APP_SWIPE_TYPE_UP)) {
            this.swipeToUp(androidDriver);
        } else if (type.equals(APP_SWIPE_TYPE_DOWN)) {
            this.swipeToDown(androidDriver);
        } else if (type.equals(APP_SWIPE_TYPE_LEFT)) {
            this.swipeToLeft(androidDriver);
        } else if (type.equals(APP_SWIPE_TYPE_RIGHT)) {
            this.swipeToRight(androidDriver);
        }
    }

    private List<WebElement> getElementsByType(AndroidDriver androidDriver, String findType, String findKey) {
        List<WebElement> elements = null;
        try {
            if (findType.equals(FIND_ELEMENT_TYPE_ID)) {
                elements = androidDriver.findElementsById(findKey);
            }
            if (findType.equals(FIND_ELEMENT_TYPE_CLASS)) {
                elements = androidDriver.findElementsByClassName(findKey);
            }
            if (findType.equals(FIND_ELEMENT_TYPE_XPATH)) {
                elements = androidDriver.findElementsByXPath(findKey);
            }
        } catch (Exception e) {
            log.error("元素不存在：findType：{}，keywords:{}", findType, findKey);
        }
        return elements;
    }
}
