package cn.com.ctop.toutiao.service.impl;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.common.module.utils.PropertiesUtils;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.toutiao.entity.*;
import cn.com.ctop.toutiao.mapper.*;
import cn.com.ctop.toutiao.service.IByteDanceAdvertisePlanService;
import cn.com.ctop.toutiao.service.IByteDanceAdvertiserDataService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author 宋英豪
 */
@Slf4j
@Service
public class ByteDanceAdvertiserDataServiceImpl implements IByteDanceAdvertiserDataService {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private ByteDanceAdvertisePlanMapper advertisePlanMapper;
    @Autowired
    private ByteDanceCampaignTemplateMapper campaignTemplateMapper;
    @Autowired
    private ByteDanceCustomAudienceMapper audienceMapper;
    @Autowired
    private ByteDanceCreativeMaterialMapper creativeMaterialMapper;
    @Autowired
    private ByteDanceCreativeMapper creativeMapper;
    @Autowired
    private ByteDanceAdvertiserBudgetMapper advertiserBudgetMapper;
    @Autowired
    private ByteDanceCampaignMapper campaignMapper;
    @Autowired
    private ByteDanceAdvertiserMapper advertiserMapper;

    @Override
    public Map<String, Object> getAdvertiserInfo(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_advertiser_info");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());
        TreeMap<String, Object> params = new TreeMap<>();
        //TODO jsonArray
        params.put("advertiser_ids", "[" + cTopOauthToken.getAccountId() + "]");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("获取广告主信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", "获取广告主信息接口异常");
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONArray("data");
        if (null == data || data.size() <= 0) {
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", "广告主信息不存在");
            return resultMap;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceAdvertiser advertiser = new ByteDanceAdvertiser(dataObject, accountId);
            //清除清数据，插入新数据 TODO
            advertiserMapper.deleteById(advertiser.getId());
            advertiserMapper.insert(advertiser);
            resultMap.put("data", advertiser);
        }
        resultMap.put("success", true);
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告主信息成功");
        return resultMap;
    }

    public void getAd(String accountId, int pageNum, String ids) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId);
        String[] idString = ids.split(",");
        JSONArray getIds = new JSONArray();
        if (null != idString && idString.length > 0) {
            for (int i = 0; i < idString.length; i++) {
                Long getId = Long.parseLong(idString[i]);
                getIds.add(getId);
            }
        }

        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/ad/get/";
        // 请求参数
        Map<String, Object> filtering = new HashMap<>();
        filtering.put("ids", getIds);

        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page", pageNum);
        param.put("page_size", 100);
        param.put("filtering", filtering);
        log.info("查询广告计划信息参数：{}", param.toJSONString());
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, param);
        log.info("获取公告计划信息返回数据：{}", resultObject.toJSONString());
        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告计划信息接口异常==》accountId:{},message:{}", accountId, resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.size() <= 0) {
            log.error("获取广告计划信息不存在==》accountId:{},message:{}", accountId, resultObject.getString("message"));
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceAdvertisePlan advertisePlan = new ByteDanceAdvertisePlan(dataObject, accountId);
            //清除清数据，插入新数据
            advertisePlanMapper.deleteById(advertisePlan.getId());
            advertisePlanMapper.insert(advertisePlan);
        }
        getAd(accountId, pageNum + 1, ids);
    }

    @Override
    public Map<String, Object> getAdvertiserPlan(String accountId, String ids) {
        getAd(accountId, 1, ids);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告计划信息成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserCampaign(String accountId, String ids) {
        getAdvertiserCampaignByPageNumber(accountId, 1, ids);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告組信息成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserBudget(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        params.put("advertiser_ids", "[" + cTopOauthToken.getAccountId() + "]");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("获取广告预算信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "获取广告预算信息接口异常");
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONArray("data");
        if (null == data || data.size() <= 0) {
            log.info("广告主预算信息不存在==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", 0);
            resultMap.put("message", "获取广告主预算信息完成");
            return resultMap;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceAdvertiserBudget advertiserBudget = new ByteDanceAdvertiserBudget(dataObject, accountId);
            //清除清数据，插入新数据
            advertiserBudgetMapper.deleteById(advertiserBudget.getId());
            advertiserBudgetMapper.insert(advertiserBudget);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告预算信息完成");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserCreative(String accountId, String ids) {
        Map<String, Object> resultMap = new HashMap<>();
        getAdvertiserCreativeByPageNumber(accountId, 1, ids);
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告主广告创意信息完成");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserCreativeMaterial(String accountId, String creativeIds) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_creative_material_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        if (null != creativeIds && !"".equals(creativeIds.trim())) {
            params.put("creative_ids", "[" + creativeIds + "]");
        }
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("获取广告主创意素材信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "获取广告主创意素材信息接口异常");
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONArray("data");
        if (null == data || data.size() <= 0) {
            log.info("广告主创意素材信息不存在==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", 0);
            resultMap.put("message", "获取广告主预算信息完成");
            return resultMap;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceCreativeMaterial material = new ByteDanceCreativeMaterial(dataObject, accountId);
            //清除清数据，插入新数据
            creativeMaterialMapper.deleteById(material.getId());
            creativeMaterialMapper.insert(material);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告主预算信息完成");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserCampaignUpdateStatus(String accountId, String campaignIds, String optStatus) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray ids = new JSONArray();
        String[] getIds = campaignIds.split(",");
        if (null != getIds && getIds.length > 0) {
            for (int i = 0; i < getIds.length; i++) {
                ids.add(Long.parseLong(getIds[i]));
            }
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_update_status");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("campaign_ids", ids);
        params.put("opt_status", optStatus);
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("广告组更新状态接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新状态接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告组更新状态异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新状态异常");
            return resultMap;
        }
        JSONArray getCampaignIds = data.getJSONArray("campaign_ids");
        if (null != getCampaignIds && getCampaignIds.size() > 0) {
            for (int i = 0; i < getCampaignIds.size(); i++) {
                Long id = getCampaignIds.getLong(i);
                ByteDanceCampaign campaign = campaignMapper.selectById(id);
                if (null != campaign) {
                    campaign.setOptStatus(optStatus);
                }
                campaignMapper.updateById(campaign);
            }
        }
        resultMap.put("code", 0);
        resultMap.put("message", "广告组状态修改成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserCampaignUpdate(String accountId, Long campaignId, String budgetMode, Integer budget, String campaignName) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        ByteDanceCampaign campaign = campaignMapper.selectById(campaignId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_update");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("campaign_id", campaignId);
        params.put("modify_time", campaign.getModifyTime());
        if (null != campaignName && !"".equals(campaignName.trim())) {
            params.put("campaign_name", campaignName);
        }
        if (null != budgetMode && !"".equals(budgetMode.trim())) {
            params.put("budget_mode", budgetMode);
        }
        if (null != budget) {
            params.put("budget", budget);
        }
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("广告组更新接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告组更新异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新异常");
            return resultMap;
        }
        if (null != campaignName && !"".equals(campaignName.trim())) {
            campaign.setName(campaignName);
        }
        if (null != budgetMode && !"".equals(budgetMode.trim())) {
            params.put("budget_mode", budgetMode);
            campaign.setBudgetMode(budgetMode);
        }
        if (null != budget) {
            campaign.setBudget(new BigDecimal(budget));
        }
        campaignMapper.updateById(campaign);

        resultMap.put("code", 0);
        resultMap.put("message", "广告组信息修改成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateStatus(String accountId, String adIds, String optStatus) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray ids = new JSONArray();
        String[] getIds = adIds.split(",");
        if (null != getIds && getIds.length > 0) {
            for (int i = 0; i < getIds.length; i++) {
                ids.add(Long.parseLong(getIds[i]));
            }
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_status");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("ad_ids", ids.toJSONString());
        params.put("opt_status", optStatus);
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.error("广告计划更新状态接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告计划更新状态接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告计划更新状态异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告计划更新状态异常");
            return resultMap;
        }
        JSONArray getAdIds = data.getJSONArray("ad_ids");
        if (null != getAdIds && getAdIds.size() > 0) {
            //获取广告计划信息
            getAdvertiserPlan(accountId, adIds);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "广告组状态修改成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateBid(String accountId, String adIds, String bids) {
        //TODO 校验数据是否完整
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray data = new JSONArray();
        String[] getadIds = adIds.split(",");
        String[] getBids = bids.split(",");
        if (null != getadIds && getadIds.length > 0) {
            for (int i = 0; i < getadIds.length; i++) {
                JSONObject object = new JSONObject();
                Long adId = Long.parseLong(getadIds[i]);
                Long bid = Long.parseLong(getBids[i]);
                object.put("ad_id", adId);
                object.put("bid", bid);
                data.add(object);
            }
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_bid");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("data", data.toJSONString());
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("修改计划出价接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划出价接口异常");
            return resultMap;
        }
        JSONObject getData = jsonObject.getJSONObject("data");
        if (null == getData) {
            log.info("修改计划出价异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划出价异常");
            return resultMap;
        }
        JSONArray getAdIds = getData.getJSONArray("ad_ids");
        if (null != getAdIds && getAdIds.size() > 0) {
            //获取广告计划信息
            getAdvertiserPlan(accountId, adIds);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "修改计划出价成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserPlanUpdateBudget(String accountId, String adIds, String budgets) {
        //TODO 校验数据是否完整
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray data = new JSONArray();
        String[] getadIds = adIds.split(",");
        String[] getBudgets = budgets.split(",");
        if (null != getadIds && getadIds.length > 0) {
            for (int i = 0; i < getadIds.length; i++) {
                JSONObject object = new JSONObject();
                Long adId = Long.parseLong(getadIds[i]);
                Long budget = Long.parseLong(getBudgets[i]);
                object.put("ad_id", adId);
                object.put("budget", budget);
                data.add(object);
            }
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_update_budget");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("data", data.toJSONString());
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.info("修改计划预算接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划预算接口异常");
            return resultMap;
        }
        JSONObject getData = jsonObject.getJSONObject("data");
        if (null == getData) {
            log.info("修改计划预算异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改计划预算异常");
            return resultMap;
        }
        JSONArray getAdIds = getData.getJSONArray("ad_ids");
        if (null != getAdIds && getAdIds.size() > 0) {
            //获取广告计划信息
            getAdvertiserPlan(accountId, adIds);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "修改计划预算成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserCreativeUpdateStatus(String accountId, String creativeIds, String optStatus) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray ids = new JSONArray();
        String[] getCreativeIds = creativeIds.split(",");
        if (null != getCreativeIds && getCreativeIds.length > 0) {
            for (int i = 0; i < getCreativeIds.length; i++) {
                ids.add(Long.parseLong(getCreativeIds[i]));
            }
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_creative_update_status");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("creative_ids", ids.toJSONString());
        params.put("opt_status", optStatus);
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("修改创意状态接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改创意状态接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("修改创意状态异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "修改创意状态异常");
            return resultMap;
        }
        JSONArray returnCreativeIds = data.getJSONArray("creative_ids");
        if (null != returnCreativeIds && returnCreativeIds.size() > 0) {
            getAdvertiserCreative(accountId, creativeIds);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "广告创意状态修改成功");
        return resultMap;
    }

    /**
     * 查询人群包信息
     *
     * @return
     */
    @Override
    public Map<String, Object> advertiserCustomAudienceSelect(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId);

        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_dmp_custom_audience_select");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        params.put("select_type", "0");
        params.put("advertiser_id", token.getAccountId() + "");
        params.put("limit", "100");
        params.put("offset", "0");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("获取人群包信息接口异常==》accountId:{},message:{}", token.getAdvertiserId(), jsonObject.getString("message"));
            resultMap.put("success", false);
            resultMap.put("message", "获取人群包信息接口异常");
            resultMap.put("code", -1);
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONObject("data").getJSONArray("custom_audience_list");
        if (null == data || data.size() <= 0) {
            log.info("人群包信息不存在==》accountId:{},message:{}", token.getAdvertiserId(), jsonObject.getString("message"));
            resultMap.put("success", false);
            resultMap.put("message", "人群包信息不存在");
            resultMap.put("code", -1);
            return resultMap;
        }
        resultMap.put("success", true);
        resultMap.put("message", "人群包信息获取成功");
        resultMap.put("data", data);
        resultMap.put("code", 0);
        return resultMap;
    }

    @Override
    public Map<String, Object> advertiserAdUpdate(String accountId, Long adId, ByteDanceUserOrientationTemplate template) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId);
        return null;
    }

    @Override
    public JSONObject updateAd(CtopOauthToken token, Long adId, ByteDanceUserOrientationTemplate template, JSONObject requestJson) {

        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/ad/update/";
        // 请求参数
        JSONObject data = new JSONObject();
        data.put("advertiser_id", token.getAccountId());
        data.put("ad_id", adId);
        data = setUserOrentationData(data, template, adId, requestJson);
        return HttpUtils.bytedancePostRequest(token.getAccessToken(), url, data);
    }

    @Autowired
    private IByteDanceAdvertisePlanService advertisePlanService;

    @Override
    public JSONObject setUserOrentationData(JSONObject data, ByteDanceUserOrientationTemplate template, Long adId, JSONObject requestJson) {
        //根据adId获取公告计划信息
        ByteDanceAdvertisePlan advertisePlan = advertisePlanService.getById(adId);
        data.put("modify_time", advertisePlan.getModifyTime());
        Integer deliveryType = requestJson.getInteger("deviceType");
        if (null != deliveryType && deliveryType == 2) {
            //穿山甲
            //精选流量包类型
            String superiorPopularityType = template.getSuperiorPopularityType();
            if (null != superiorPopularityType && "ZDY".equals(superiorPopularityType)) {
                JSONArray flowPackage = requestJson.getJSONArray("flowPackage");
                if (null != flowPackage && flowPackage.size() > 0) {
                    JSONArray getFlowPackage = new JSONArray();
                    for (int i = 0; i < flowPackage.size(); i++) {
                        String flowPackageString = flowPackage.getString(i);
                        getFlowPackage.add(Long.parseLong(flowPackageString));
                    }
                    data.put("flow_package", getFlowPackage);
                }
                JSONArray excludeFlowPackage = requestJson.getJSONArray("excludeFlowPackage");
                if (null != excludeFlowPackage && excludeFlowPackage.size() > 0) {
                    JSONArray getExcludeFlowPackage = new JSONArray();
                    for (int i = 0; i < excludeFlowPackage.size(); i++) {
                        String excludeFlowPackageString = excludeFlowPackage.getString(i);
                        getExcludeFlowPackage.add(Long.parseLong(excludeFlowPackageString));
                    }
                    data.put("exclude_flow_package", getExcludeFlowPackage);
                }
            } else {
                data.put("superior_popularity_type", superiorPopularityType);
            }
            //设备类型
            String deviceType = template.getDeviceType();
            if (null != deviceType && !"".equals(deviceType) && !"NONE".equals(deviceType)) {
                JSONArray typeArray = new JSONArray();
                typeArray.add(deviceType);
                data.put("device_type", typeArray);
            }
        }

        JSONArray retargetingTagsExclude = requestJson.getJSONArray("retargetingTagsExclude");
        JSONArray retargetingTagsInclude = requestJson.getJSONArray("retargetingTagsInclude");
        if (null != retargetingTagsExclude && retargetingTagsExclude.size() > 0) {
            JSONArray getRetargetingTagsExclude = new JSONArray();
            for (int i = 0; i < retargetingTagsExclude.size(); i++) {
                String exclude = retargetingTagsExclude.getString(i);
                getRetargetingTagsExclude.add(Long.parseLong(exclude));
            }
            data.put("retargeting_tags_exclude", getRetargetingTagsExclude);
        }
        if (null != retargetingTagsInclude && retargetingTagsInclude.size() > 0) {
            JSONArray getRetargetingTagsInclude = new JSONArray();
            for (int i = 0; i < retargetingTagsInclude.size(); i++) {
                String include = retargetingTagsInclude.getString(i);
                getRetargetingTagsInclude.add(Long.parseLong(include));
            }
            data.put("retargeting_tags_include", getRetargetingTagsInclude);
        }
        data = setUserOrentationData(data, template);
        return data;
    }

    @Override
    public JSONObject setUserOrentationData(JSONObject data, ByteDanceUserOrientationTemplate template) {
        //根据adId获取公告计划信息
        if (null != template.getGender() && !"".equals(template.getGender().trim())) {
            data.put("gender", template.getGender());
        }
        if (null != template.getAge() && !"".equals(template.getAge().trim())) {
            JSONArray ageArray = JSONArray.parseArray(template.getAge());
            data.put("age", ageArray);
        }
        if (null != template.getAndroidOsv() && !"".equals(template.getAndroidOsv().trim())) {
            data.put("android_osv", template.getAndroidOsv());
        }
        if (null != template.getIosOsv() && !"".equals(template.getIosOsv().trim())) {
            data.put("ios_osv", template.getIosOsv());
        }
        if (null != template.getCarrier() && !"".equals(template.getCarrier().trim())) {
            JSONArray carrierArray = JSONArray.parseArray(template.getCarrier());
            data.put("carrier", carrierArray);
        }
        if (null != template.getAc() && !"".equals(template.getAc().trim())) {
            JSONArray acArray = JSONArray.parseArray(template.getAc());
            data.put("ac", acArray);
        }
        if (null != template.getDeviceBrand() && !"".equals(template.getDeviceBrand().trim())) {
            JSONArray deviceBrandArray = JSONArray.parseArray(template.getDeviceBrand());
            data.put("device_brand", deviceBrandArray);
        }
        if (null != template.getArticleCategory() && !"".equals(template.getArticleCategory().trim())) {
            JSONArray articleCategoryArray = JSONArray.parseArray(template.getArticleCategory());
            data.put("article_category", articleCategoryArray);
        }
        if (null != template.getActivateType() && !"".equals(template.getActivateType().trim())) {
            JSONArray activateTypeArray = JSONArray.parseArray(template.getActivateType());
            data.put("activate_type", activateTypeArray);
        }
        if (null != template.getPlatform() && !"".equals(template.getPlatform().trim())) {
            JSONArray platformArray = new JSONArray();
            platformArray.add(template.getPlatform());
            data.put("platform", platformArray);
        }
        if (null != template.getBusinessIds() && !"".equals(template.getBusinessIds().trim())) {
            JSONArray bussinessIdArray = JSONArray.parseArray(template.getBusinessIds());
            data.put("business_ids", bussinessIdArray);
        }
        if (null != template.getAreaType() && !"".equals(template.getAreaType().trim())) {
            data.put("district", template.getAreaType());
            if ("CITY".equals(template.getAreaType())) {
                JSONArray cityArray = JSONArray.parseArray(template.getCity());
                data.put("city", cityArray);
            } else if ("COUNTY".equals(template.getAreaType())) {
                JSONArray countyArray = JSONArray.parseArray(template.getDistrict());
                data.put("city", countyArray);
            }

        }

        if (null != template.getLocationType() && !"".equals(template.getLocationType().trim())) {
            data.put("location_type", template.getLocationType());
        }
        if (null != template.getAdTag() && !"".equals(template.getAdTag().trim())) {
            JSONArray adTagArray = JSONArray.parseArray(template.getAdTag());
            data.put("ad_tag", adTagArray);
        }
        if (null != template.getInterestTags() && !"".equals(template.getInterestTags().trim())) {
            JSONArray interestTagArray = JSONArray.parseArray(template.getInterestTags());
            data.put("interest_tags", interestTagArray);
        }
        if (null != template.getAppBehaviorTarget() && !"".equals(template.getAppBehaviorTarget().trim())) {
            data.put("app_behavior_target", template.getAppBehaviorTarget());
        }
        if (null != template.getAppCategory() && !"".equals(template.getAppCategory().trim())) {
            JSONArray appCategoryArray = JSONArray.parseArray(template.getAppCategory());
            data.put("app_category", appCategoryArray);
        }
        if (null != template.getAppIds() && !"".equals(template.getAppIds().trim())) {
            JSONArray appIdsArray = JSONArray.parseArray(template.getAppIds());
            data.put("app_ids", appIdsArray);
        }
        if (null != template.getProductPlatformId() && !"".equals(template.getProductPlatformId().trim())) {
            data.put("product_platform_id", template.getProductPlatformId());
        }
        if (null != template.getExternalUrlParams() && !"".equals(template.getExternalUrlParams().trim())) {
            data.put("external_url_params", template.getExternalUrlParams());
        }
        if (null != template.getOpenUrlParams() && !"".equals(template.getOpenUrlParams().trim())) {
            data.put("open_url_params", template.getOpenUrlParams());
        }
        if (null != template.getDpaLocalAudience() && !"".equals(template.getDpaLocalAudience().trim())) {
            data.put("dpa_local_audience", Long.parseLong(template.getDpaLocalAudience()));
        }
        if (null != template.getExcludeCustomActions() && !"".equals(template.getExcludeCustomActions().trim())) {
            JSONArray excludeCustomActionsArray = JSONArray.parseArray(template.getExcludeCustomActions());
            data.put("exclude_custom_actions", excludeCustomActionsArray);
        }
        if (null != template.getIncludeCustomActions() && !"".equals(template.getIncludeCustomActions().trim())) {
            JSONArray includeCustomActionsArray = JSONArray.parseArray(template.getIncludeCustomActions());
            data.put("include_custom_actions", includeCustomActionsArray);
        }
        if (null != template.getSuperiorPopularityType() && !"".equals(template.getSuperiorPopularityType().trim())) {
            data.put("superior_popularity_type", template.getSuperiorPopularityType());
        }
        if (null != template.getFlowPackage() && !"".equals(template.getFlowPackage().trim())) {
            JSONArray flowPackageArray = JSONArray.parseArray(template.getFlowPackage());
            data.put("flow_package", flowPackageArray);
        }
        if (null != template.getExcludeFlowPackage() && !"".equals(template.getExcludeFlowPackage().trim())) {
            JSONArray excludeFlowPackageArray = JSONArray.parseArray(template.getExcludeFlowPackage());
            data.put("exclude_flow_package", excludeFlowPackageArray);
        }
        if (null != template.getAutoExtendEnabled() && !"".equals(template.getAutoExtendEnabled().trim())) {
            data.put("auto_extend_enabled", template.getAutoExtendEnabled());
        }
        return data;
    }

    @Override
    public Map<String, Object> getAdvertiserCampaignList(String accountId) {
        Map<String, Object> resultMap = new HashMap<>();
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("page", 1);
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.error("获取广告组信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", code);
            resultMap.put("message", jsonObject.getString("message"));
            return resultMap;
        }
        JSONArray data = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.size() <= 0) {
            log.info("获取广告组信息为空==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            resultMap.put("code", code);
            resultMap.put("message", "获取广告组信息为空");
            return resultMap;
        }
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        resultMap.put("data", data);
        return resultMap;
    }

    /**
     * 获取流量包数据接口
     *
     * @param accountId
     * @return
     */
    @Override
    public Map<String, Object> flowPackageGet(String accountId) {
        JSONObject jsonObject = flowPackageGetJSONObject(accountId);
        JSONObject result = new JSONObject();
        Integer code = jsonObject.getInteger("code");
        String message = jsonObject.getString("message");
        result.put("code", code);
        result.put("message", message);
        if (null == code || code != 0) {
            result.put("success", false);
            return result;
        }
        JSONArray packageList = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null != packageList && packageList.size() > 0) {
            JSONArray packageArray = new JSONArray();
            for (int i = 0; i < packageList.size(); i++) {
                JSONObject packageObject = packageList.getJSONObject(i);
                packageObject.remove("rit");
                String status = packageObject.getString("status");
                if (null != status && status.equals("FLOW_PACKAGE_ENABLE")) {
                    packageArray.add(packageObject);
                }
            }
            result.put("data", packageArray);
        }
        result.put("success", true);
        return result;
    }


    public JSONObject flowPackageGetJSONObject(String accountId) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId);
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/union/flow_package/get/";
        // 请求参数
        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("page", 1);
        params.put("page_size", 100);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, params);
        return resultObject;
    }


    private void getAdvertiserCreativeByPageNumber(String accountId, Integer pageNumber, String ids) {
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_creative_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        JSONObject filtering = new JSONObject();
        if (null != ids && !"".equals(ids.trim())) {
            String[] idsArray = ids.split(",");
            JSONArray filterIdsArray = new JSONArray();
            for (int i = 0; i < idsArray.length; i++) {
                Long id = Long.parseLong(idsArray[i]);
                filterIdsArray.add(id);
            }
            filtering.put("ids", filterIdsArray);
        }
        params.put("filtering", filtering.toJSONString());
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("page", pageNumber + "");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("获取广告主广告创意信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            return;
        }
        JSONArray data = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.size() <= 0) {
            log.info("广告主广告创意信息不存在==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceCreative creative = new ByteDanceCreative(dataObject, accountId);
            creativeMapper.deleteById(creative.getId());
            creativeMapper.insert(creative);
        }
        getAdvertiserCreativeByPageNumber(accountId, pageNumber + 1, ids);
    }

    private void getAdvertiserCampaignByPageNumber(String accountId, Integer pageNumber, String ids) {
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", cTopOauthToken.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        JSONObject filtering = new JSONObject();
        if (null != ids && !"".equals(ids.trim())) {
            String[] idsArray = ids.split(",");
            JSONArray filterIdsArray = new JSONArray();
            for (int i = 0; i < idsArray.length; i++) {
                Long id = Long.parseLong(idsArray[i]);
                filterIdsArray.add(id);
            }
            filtering.put("ids", filterIdsArray);
        }
        params.put("filtering", filtering.toJSONString());
        params.put("advertiser_id", cTopOauthToken.getAccountId());
        params.put("page", pageNumber + "");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.error("获取广告组信息接口异常==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            return;
        }
        JSONArray data = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.size() <= 0) {
            log.error("获取广告组信息不存在==》accountId:{},message:{}", accountId, jsonObject.getString("message"));
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceCampaign campaign = new ByteDanceCampaign(dataObject, accountId);
            campaignMapper.deleteById(campaign.getId());
            campaignMapper.insert(campaign);
        }
        getAdvertiserCampaignByPageNumber(accountId, pageNumber + 1, ids);
    }
}
